unit fb_descriptors;

interface

uses ib_externals;

const
  DSC_null = 1;
  dtype_null = 0;
  dtype_text = 1;
  dtype_cstring = 2;
  dtype_varying = 3;
  dtype_packed = 6;
  dtype_byte = 7;
  dtype_short = 8;
  dtype_long = 9;
  dtype_quad = 10;
  dtype_real = 11;
  dtype_double = 12;
  dtype_d_float = 13;
  dtype_sql_date = 14;
  dtype_sql_time = 15;
  dtype_timestamp = 16;
  dtype_blob = 17;
  dtype_array = 18;
  dtype_int64 = 19;
  dtype_TYPE_MAX = 20;

type
  ParamDsc = record
    dsc_dtype: Byte;
    dsc_scale: ShortInt;
    dsc_length: Word;
    dsc_sub_type: SmallInt;
    dsc_flags: Word;
    dsc_address: Pointer;
  end;
  PParamDsc = ^ParamDsc;

  ISC_INT64 = Int64;
  PISC_INT64 = ^ISC_INT64;

procedure Set_Double_Type(P: PParamDsc; const Value: Double);
function get_double_type(v: PParamDsc; var rc: double): smallint;
function get_int_type(v: PParamDsc; var rc: Int64): smallint;
function get_scaled_double(v: PParamDsc; var rc: double): smallint;

implementation

procedure Set_Double_Type(P: PParamDsc; const Value: Double);
var
  S: Single;
begin
  case P.dsc_dtype of
    dtype_real:
      begin
        S := Value;
        PFloat(P.dsc_address)^ := S;
      end;
    dtype_double: PDouble(P.dsc_address)^ := Value;
  end;
end;

function get_double_type(v: PParamDsc; var rc: double): smallint;
begin
  result := -1;
  case v.dsc_dtype of
    dtype_real:
      begin
        rc := PFloat(v.dsc_address)^;
        result := sizeof(float);
      end;
    dtype_double:
      begin
        rc := PDouble(v.dsc_address)^;
        result := sizeof(double);
      end;
  end;
end;

function get_int_type(v: PParamDsc; var rc: Int64): smallint;
begin
  Result := -1;
  case v.dsc_dtype of
    dtype_short:
      begin
        rc := PShort(v.dsc_address)^;
        result := sizeof(short);
      end;
    dtype_long:
      begin
        rc := PISC_LONG(v.dsc_address)^;
        result := sizeof(ISC_LONG);
      end;
    dtype_int64:
      begin
        rc := PISC_INT64(v.dsc_address)^;
        result := sizeof(ISC_INT64);
      end;
  end;
end;

function get_scaled_double(v: PParamDsc; var rc: double): smallint;
var
  iv: ISC_INT64;
  i, scale: Smallint;
begin
  result := get_int_type(v, iv);
  if (result < 0) then
    result := get_double_type(v, rc)
  else
  begin
    rc := iv;
    scale := v.dsc_scale;
    if scale < 0 then
      for i := scale to -1 do rc := rc / 10
    else
      if scale > 0 then
        for i := scale downto 1 do rc := rc * 10;
  end;
end;

end.

